package anynt.environment.settings;


import anynt.object.Agent;
import anynt.object.EvaluableAgent;
import anynt.space.Cell;


/**
 * Represents the interface of a reward provider.
 * This provider shares the rewards between all the agents in the cell, so all the agents
 * obtains the same reward divided by the number of agents that are in the cell.
 * 
 * @author Javier
 * @since 11/04/2011
 */
public class ShareRewardsBetweenEvaluableAgents implements RewardsProvider
{
	/**
	 * It returns the proportional reward for each agent that is located in the cell
	 */
	public double providedReward(Cell cell)
	{
		int evaluableAgents = 0;

		for (Agent agent : cell.getAgents())
			if (agent instanceof EvaluableAgent)
				evaluableAgents++;

		if (evaluableAgents == 0)
			return 0.0;
		return cell.getReward() / evaluableAgents;
	}
}