package anynt.object.behaviour;


import anynt.space.Observation;


/**
 * It extends the behavior of an agent to represent the behavior of a human
 * 
 * @author Javier
 * @since 31/03/2011
 */
public class HumanBehaviour extends AgentBehaviour
{
	// Object attributes
	/**
	 * Action that the human will do
	 * It's necessary to keep to synchronize between threads
	 */
	private int action;


	// Constructors
	public HumanBehaviour()
	{
		action = 0;
	}

	// Object overrides
	public HumanBehaviour clone()
	{
		return new HumanBehaviour();
	}

	// AgentBehavior overrides
	public synchronized int nextAction(Observation observation)
	{
		long initialTime = 0;
		long finalTime = 0;

		initialTime = System.currentTimeMillis();
		// Wait's for the Human action
		try
		{
			wait();
		}
		catch (Exception e)
		{
			action = 0;
		}

		finalTime = System.currentTimeMillis();
		// Actualizes the time that the agent will spend to realize the action
		elapsedTime += finalTime - initialTime;

		return action;
	}

	// Treatment of object attributes
	public synchronized void setAction(int action)
	{
		// If the action is able it will be made, otherwise it will remain on the cell
		this.action = availableActions().contains(action) ? action : 0;

		notify();
	}
}