package anynt.object.behaviour;


import java.util.Random;

import distribution.IDistribution;
import distribution.UniformDistribution;

import anynt.space.Observation;


/**
 * It extends the behavior of an agent to represent the behavior following a pattern
 * 
 * @author Javier
 * @since 31/03/2011
 */
public class PatternBehaviour extends AgentBehaviour
{
	// Class functions
	public static String generatePattern(int[] actions, int stopProbability)
	{
		String pattern = "";
		IDistribution uniformDistribution = new UniformDistribution();

		// It constructs a pattern from some actions
		do
			pattern += actions[uniformDistribution.getValue(actions.length)];
		while (uniformDistribution.getValue(1.0) > 1.0 / stopProbability);

		return pattern;
	}


	// Object attributes
	/**
	 * Action pattern to follow
	 */
	private final String pattern;

	/**
	 * Actual state (ndex) of the pattern
	 * It indicates in what part of the pattern we are
	 */
	private int state;


	// Constructors
	public PatternBehaviour(String pattern)
	{
		this.pattern = pattern;
		state = 0;
	}

	// Object overrides
	public PatternBehaviour clone()
	{
		PatternBehaviour behaviour = new PatternBehaviour(pattern);
		behaviour.state = state;

		return behaviour;
	}

	// AgentBehaviour overrides
	/**
	 * Reset the agent behavior to his initial definition
	 */
	public void reset()
	{
		super.reset();

		state = 0;
	}

	/**
	 * It returns the next action realized by the agent
	 */
	public int nextAction(Observation observation)
	{
		// Actualizes the time that is spend to realize the action
		long maximumTime = this.agent.getMaximumTimeToAct();
		elapsedTime += new Random().nextLong() % (maximumTime + 1);

		// It gets the next action to make
		int action = Character.getNumericValue(pattern.charAt(state));
		// It actualizes pattern state
		state++;
		if (state >= pattern.length())
			state = 0;

		return action;
	}

	// Treatment of object attributes
	public int patternLenght()
	{
		return pattern.length();
	}
	public String getPattern()
	{
		return pattern;
	}
	public int getState()
	{
		return state;
	}
}