package anynt.object.behaviour;

import java.util.ArrayList;
import java.util.Random;
import java.util.TreeMap;

import anynt.object.Agent;
import anynt.space.Cell;
import anynt.space.Observation;
import anynt.space.Space;

public class _OptimisticBehaviour extends AgentBehaviour
{
	private ArrayList<Agent> agents;
	private String lastStateDescription;
	private String stateDescription;
	private int lastAction;
	private TreeMap<String, double[][]> matrix;


	public _OptimisticBehaviour(ArrayList<Agent> agents)
	{
		this.agents = agents;

		matrix = new TreeMap<String, double[][]>();
	}
	public AgentBehaviour clone()
	{
		return new _OptimisticBehaviour(agents);
	}


	public int nextAction(Observation observation)
	{
		Space space = (Space) observation.getObservation();
		if (stateDescription == null)
			stateDescription = getStateDescription(space);
		lastAction = policy();
		return lastAction;
	}
	public void actualize()
	{
		Space space = (Space) agent.getActualInteraction().getObservation().getObservation();
		lastStateDescription = stateDescription;
		stateDescription = getStateDescription(space);

		updateReward(lastStateDescription, lastAction, agent.getActualInteraction().getReward());
	}


	private int policy()
	{
		double max = Double.NEGATIVE_INFINITY;
		double maxViews = Double.NEGATIVE_INFINITY;
		ArrayList<Integer> bestActions = new ArrayList<Integer>();

		int[] actions = agent.getKnownActions();
		if (!matrix.containsKey(stateDescription))
			addState(stateDescription);
		double[][] rewards = matrix.get(stateDescription);
		for (int index = 0; index < rewards.length; index++)
		{
			double adjustedReward = rewards[index][0];
			double viewsOfTheState = rewards[index][1];
			if (adjustedReward > max
			|| (adjustedReward == max && viewsOfTheState > maxViews))
			{
				max = adjustedReward;
				maxViews = viewsOfTheState;
				bestActions.clear();
				bestActions.add(actions[index]);
			}
			else if (adjustedReward == max && viewsOfTheState == maxViews)
				bestActions.add(actions[index]);
		}

		return bestActions.get(new Random().nextInt(bestActions.size()));
	}

	private String getStateDescription(Space space)
	{
		String description = "";

		for (Cell cell : space.getCells())
		{
			for (Agent agent : agents)
				if (cell.getAgents().contains(agent))
					description = description.concat("1");
				else
					description = description.concat("0");
			description = description.concat("|");
		}
		description = description.substring(0, description.length() - 1);

		return description;
	}
	private void addState(String state)
	{
		int actions = agent.getKnownActions().length;
		double[][] rewards = new double[actions][2];
		for (int index = 0; index < rewards.length; index++)
		{
			rewards[index][0] = +1.0;
			rewards[index][1] = 1;
		}
		matrix.put(state, rewards);
	}
	private void updateReward(String state, int action, double reward)
	{
		if (!matrix.containsKey(state))
			addState(state);
		double[][] rewards = matrix.get(state);
		int[] actions = agent.getKnownActions();
		for (int index = 0; index < actions.length; index++)
			if (actions[index] == action)
			{
				rewards[index][0] = (rewards[index][0] * rewards[index][1] + reward) / (rewards[index][1] + 1);
				rewards[index][1]++;
				break;
			}
	}
}