package distribution;


import java.util.Random;


/**
 * It implements the interface of a distribution representing a uniform distribution
 * 
 * @author Javier
 * @since 30/03/2011
 */
public class UniformDistribution implements IDistribution
{
	// Object functions
	/**
	 * It returns a random value between 0 and a maximum
	 */
	public int getValue(int max)
	{
		if (max <= 0)
			throw new IllegalArgumentException();

		Random random = new Random();
		return random.nextInt(max);
	}
	/**
	 * It returns a random value between a minimum and a maximum
	 */
	public int getValue(int min, int max)
	{
		if (min > max || min <= 0)
			throw new IllegalArgumentException();

		Random random = new Random();
		return random.nextInt(max - min) + min;
	}
	/**
	 * It returns a random value between 0 and a maximum
	 */
	public double getValue(double max)
	{
		if (max <= 0.0)
			throw new IllegalArgumentException();

		Random random = new Random();
		return random.nextDouble() * max;
	}
	/**
	 * It returns a random value between a minimum and a maximum
	 */
	public double getValue(double min, double max)
	{
		if (min > max || min <= 0.0)
			throw new IllegalArgumentException();

		Random random = new Random();
		return random.nextDouble() * (max - min) + min;
	}
}