package distribution;


import java.util.Random;


/**
 * It implements the interface of a distribution representing a universal distribution
 * 
 * @author Javier
 * @since 30/03/2011
 */
public class UniversalDistribution implements IDistribution
{
	// Object functions
	/**
	 * It returns a random value following a universal distribution starting from a minimum
	 */
	public int getValue(int min)
	{
		if (min <= 0)
			throw new IllegalArgumentException();

		Random random = new Random();
		int number = min;

		while (random.nextBoolean())
			number++;

		return number;
	}
	/**
	 * It returns a random value following a universal distribution starting from a minimum and with a maximum number
	 */
	public int getValue(int min, int max)
	{
		if (min > max)
			throw new IllegalArgumentException();

		int number = getValue(min);

		if (number > max)
			number = max;

		return number;
	}
	/**
	 * It returns a random value following a universal distribution starting from a minimum
	 * It increments by +1.0
	 */
	public double getValue(double min)
	{
		return getValueWithIncrement(min, +1.0);
	}
	/**
	 * It returns a random value following a universal distribution starting from a minimum
	 */
	public double getValueWithIncrement(double min, double increment)
	{
		if (min <= 0.0)
			throw new IllegalArgumentException();

		Random random = new Random();
		double number = min;

		while (random.nextBoolean())
			number += increment;

		return number;
	}
	/**
	 * It returns a random value following a universal distribution starting from a minimum to a maximum
	 * It increments by +1.0
	 */
	public double getValue(double min, double max)
	{
		return getValueWithIncrement(min, max, +1.0);
	}
	/**
	 * It returns a random value following a universal distribution starting from a minimum to a maximum
	 */
	public double getValueWithIncrement(double min, double max, double increment)
	{
		if (min > max)
			throw new IllegalArgumentException();

		double number = getValueWithIncrement(min, increment);

		if (number > max)
			number = max;

		return number;
	}
}