package interfaces;


import anynt.environment.Environment;
import anynt.environment.LambdaEnvironment;
import anynt.exercise.Exercise;
import anynt.object.EvaluableAgent;
import anynt.object.RewardsGeneratorAgent;
import anynt.space.Space;
import anynt.test.InvestigationTest;
import investigation.ExerciseOptionsPanel;
import investigation.InvestigationExercisePanel;
import investigation.InvestigationMenu;
import investigation.RewardsGeneratorsOptionsPanel;
import investigation.SpaceOptionsPanel;


@SuppressWarnings("serial")
public class InvestigationTestInterface extends TestInterface
{
	private InvestigationMenu investigationMenu;


	private Space space;
	private RewardsGeneratorAgent good;
	private RewardsGeneratorAgent evil;


	public InvestigationTestInterface()
	{
		investigationMenu = new InvestigationMenu(this);
		exercisePanel = new InvestigationExercisePanel();

		test = InvestigationTest.getTest();
		EvaluableAgent agent = new EvaluableAgent("Evaluated");
		test.setEvaluatedAgent(agent);
		Exercise exercise = new Exercise();
		exercise.setEvaluableAgent(agent);
		test.setExercise(exercise);
	}
	public static void main(String args[])
	{
		new InvestigationTestInterface().launch();
	}


	@SuppressWarnings("deprecation")
	public void launch()
	{
		setSize(WIDTH, HEIGHT);
		setTitle("Anytime Universal Intelligence - Investigation Interface");
		setResizable(false);
		setDefaultCloseOperation(EXIT_ON_CLOSE);

		setJMenuBar(investigationMenu);

		show();
	}


	public void generateEnvironment()
	{
		investigationMenu.environmentGenerated(false);
		LambdaEnvironment environment = new LambdaEnvironment();
		test.getExercise().setEnvironment(environment);
		space = new Space();
		setContentPanel(new SpaceOptionsPanel(this, space));
	}
	public void startExercise()
	{
		investigationMenu.exerciseGenerated(false);
		Exercise exercise = test.getExercise();
		Environment environment = exercise.getEnvironment();
		environment.removeAgent(exercise.getEvaluableAgent());
		exercise.reset();
		setContentPanel(new ExerciseOptionsPanel(this, test.getExercise()));
	}

	public void showInterface()
	{
		exercisePanel.notifyObserver();
	}
	public void showRewards()
	{
		exercisePanel.notifyObserver();
	}
	public void showAgents()
	{
		exercisePanel.notifyObserver();
	}


	public void agreeSpace()
	{
		LambdaEnvironment environment = (LambdaEnvironment) test.getExercise().getEnvironment();
		environment.setSpace(space);

		good = environment.getGoodAgent();
		evil = environment.getEvilAgent();
		setContentPanel(new RewardsGeneratorsOptionsPanel(this, environment, good, evil));
	}
	public void agreeRewardsGenerators()
	{
		investigationMenu.environmentGenerated(true);
		setContentPanel(null);
	}
	public void agreeExercise()
	{
		exercisePanel.setExercise(test.getExercise());
		investigationMenu.exerciseStarted(true);
		setContentPanel(exercisePanel);
		((InvestigationExercisePanel) exercisePanel).setMenu(investigationMenu);
		test.start();
	}
}