package investigation;


import interfaces.ExercisePanel;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Collection;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.border.Border;



import anynt.environment.LambdaEnvironment;
import anynt.exercise.Exercise;
import anynt.exercise.settings.StepByStepWaiting;
import anynt.object.Agent;
import anynt.object.Objeto;
import anynt.object.RewardsGeneratorAgent;
import anynt.object.behaviour.HumanBehaviour;
import anynt.object.behaviour.OracleBehaviour;
import anynt.object.behaviour.PatternBehaviour;
import anynt.object.behaviour.QLearningBehaviour;
import anynt.object.behaviour.RandomBehaviour;
import anynt.object.behaviour.TrivialFollowerBehaviour;
import anynt.space.Cell;


public class InvestigationExercisePanel extends ExercisePanel
{
	private static final long serialVersionUID = 0;


	private static boolean INITIAL_CELL = true;
	private static boolean INITIAL_CELL_AGENTS = true;
	private static boolean INITIAL_CELL_OBJECTS = true;
	private static boolean INITIAL_MOVEMENTS = true;
	private static boolean INITIAL_LAST_REWARD = true;
	private static boolean INITIAL_TOTAL_REWARD = true;

	private static final char PI = 960;
	private static final char GOOD = 8853;
	private static final char EVIL = 8854;


	private InvestigationMenu menu;

	private boolean showCell;
	private boolean showCellAgents;
	private boolean showCellObjects;
	private boolean showMovements;
	private boolean showLastReward;
	private boolean showTotalReward;

	private LambdaEnvironment environment;


	public InvestigationExercisePanel()
	{
		setLayout(null);
		setPreferredSize(new Dimension(1280, 350));

		this.initializeParameters();
	}
	public void setExercise(Exercise exercise)
	{
		super.setExercise(exercise);

		environment = (LambdaEnvironment) exercise.getEnvironment();
	}


	public void setMenu(InvestigationMenu menu)
	{
		this.menu = menu;
	}


	public void notifyObserver()
	{
		draw();
	}

	protected void clear()
	{
		removeAll();
	}
	protected void draw()
	{
		if (!(exercise.getEvaluableAgent().getBehaviour() instanceof HumanBehaviour)
			&& !menu.isInterfaceShown())
			return;

		Point startPosition = new Point(20, 20);
		Point position = new Point(20, 20);
		Point finalRewardsPosition = new Point();
		Point stepByStepPosition = new Point();
		Point agentsBehaviourPosition = new Point();
		Point rewardsGeneratorsRelocatedPosition = new Point();
		finalRewardsPosition.setLocation(startPosition);

		invalidate();
		clear();
		for (int cellNumber = 1; cellNumber <= environment.getSpace().getNumberOfCells(); cellNumber++)
		{
			JPanel panel = new JPanel();

			drawCell(panel, position, cellNumber);
			drawCellReward(panel, position, cellNumber);
			drawAgents(panel, position, cellNumber);
			drawMovements(panel, position, cellNumber);

			panel.setLayout(null);
			panel.setBounds(startPosition.x, startPosition.y, position.x + 20, position.y + 20);
			Border border = BorderFactory.createTitledBorder("Cell " + cellNumber);
			panel.setBorder(border);
			add(panel);

			if (finalRewardsPosition.getY() < startPosition.y + position.y + 40)
				finalRewardsPosition.y = startPosition.y + position.y + 40;
			startPosition.x += panel.getWidth();
			position.x = 20;
			position.y = 20;
		}
		drawLastReward(finalRewardsPosition);
		finalRewardsPosition.y += 15;
		drawTotalReward(finalRewardsPosition);

		agentsBehaviourPosition.x = finalRewardsPosition.x;
		agentsBehaviourPosition.y = finalRewardsPosition.y + 30;
		drawAgentsBehaviour(agentsBehaviourPosition);
		rewardsGeneratorsRelocatedPosition.x = agentsBehaviourPosition.x;
		rewardsGeneratorsRelocatedPosition.y = agentsBehaviourPosition.y + 50;
		drawNotifyRewardsGeneratorsRelocate(rewardsGeneratorsRelocatedPosition);
		stepByStepPosition.x = finalRewardsPosition.x + 250;
		stepByStepPosition.y = finalRewardsPosition.y - 10;
		drawStepByStep(stepByStepPosition);

		if (exercise.hasFinished())
			menu.exerciseFinished(true);

		repaint();
		validate();
	}

	private void drawCell(JPanel panel, Point position, int cellNumber)
	{
		if (!showCell)
			return;

		JPanel insidePanel = new JPanel();
		insidePanel.setLayout(null);
		insidePanel.setBounds(20, position.y, 100, 100);
		insidePanel.setBackground(new Color(200, 200, 200));

		Cell cell = environment.getSpace().getCell(cellNumber);
		Border border;
		if (cell.getAgents().contains(exercise.getEvaluableAgent()))
			border = BorderFactory.createLineBorder(new Color(0, 0, 0), 3);
		else
			border = BorderFactory.createLineBorder(new Color(0, 0, 0), 1);
		insidePanel.setBorder(border);

		Point insidePosition = new Point(5, 5);
		drawCellAgents(insidePanel, insidePosition, cellNumber);
		drawCellObjects(insidePanel, insidePosition, cellNumber);

		panel.add(insidePanel);

		position.x += insidePanel.getWidth();
		position.y += insidePanel.getHeight();
	}
	private void drawCellReward(JPanel panel, Point position, int cellNumber)
	{
		if (!menu.areRewardsShown())
			return;

		Cell cell = environment.getSpace().getCell(cellNumber);
		double reward = cell.getReward();

		JLabel label = new JLabel();
		label.setText("Reward: " + reward);
		label.setBounds(20, position.y, 100, 15);
		panel.add(label);

		position.y += label.getHeight();
	}
	private void drawAgents(JPanel panel, Point position, int cellNumber)
	{
		if (!menu.areAgentsShown())
			return;

		Cell cell = environment.getSpace().getCell(cellNumber);
		Collection<Agent> agents = cell.getAgents();

		for (Agent agent : agents)
		{
			JLabel label = new JLabel("Agent: " + agent.getName());
			label.setBounds(20, position.y, 100, 15);
			panel.add(label);

			position.y += label.getHeight();
		}
	}
	private void drawMovements(JPanel panel, Point position, int cellNumber)
	{
		if (!showMovements)
			return;

		Cell cell = environment.getSpace().getCell(cellNumber);
		Collection<Integer> actions = cell.getAvailableActions();

		JLabel labelMovements = new JLabel("Movements:");
		labelMovements.setBounds(20, position.y + 20, 100, 15);
		panel.add(labelMovements);
		position.y += labelMovements.getHeight() + 20;
		char movement = 'A';
		
		final Agent agent = exercise.getEvaluableAgent();

		for (final Integer action : actions)
		{
			JButton button = new JButton(movement + " - " + action);
			button.setBounds(20, position.y, 60, 15);
			if (agent.getBehaviour() instanceof HumanBehaviour
				&& agent.getLocation() == cell
				&& !exercise.hasFinished())
				button.addActionListener
				(
					new ActionListener()
					{
						public void actionPerformed(ActionEvent e)
						{
							((HumanBehaviour) agent.getBehaviour()).setAction(action);
						}
					}
				);
			else
				button.setEnabled(false);
			JLabel label = new JLabel("Cell " + cell.destinationCell(action).getNumber());
			label.setBounds(88, position.y, 100, 15);
			panel.add(button);
			panel.add(label);

			position.y += label.getHeight();
			movement++;
		}
	}
	private void drawLastReward(Point position)
	{
		if (!showLastReward)
			return;

		double lastReward = exercise.getEvaluableAgent().lastReward();
		JLabel label = new JLabel();
		label.setText("Last Reward: " + lastReward);
		label.setBounds(20, position.y, 250, 15);
		add(label);
	}
	private void drawTotalReward(Point position)
	{
		if (!showTotalReward)
			return;

		double totalReward = exercise.getEvaluableAgent().totalReward();
		JLabel label = new JLabel();
		label.setText("Total Reward: " + totalReward);
		label.setBounds(20, position.y, 250, 15);
		add(label);
	}
	private void drawStepByStep(Point position)
	{
		if (!(exercise.getStepWaiting() instanceof StepByStepWaiting))
			return;

		JButton button = new JButton();
		button.setText("Next step");
		button.setBounds(position.x, position.y - 5, 100, 30);
		button.addActionListener
		(
			new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					((StepByStepWaiting) exercise.getStepWaiting()).nextStep();
				}
			}
		);

		this.add(button);
	}
	private void drawAgentsBehaviour(Point position)
	{
		JLabel goodLabel = new JLabel();
		JLabel evilLabel = new JLabel();
		JLabel evaluatedLabel = new JLabel();
		goodLabel.setText("Behaviour for good: ");
		goodLabel.setLocation(20, position.y);
		evilLabel.setText("Behaviour for evil: ");
		evilLabel.setLocation(20, position.y + 15);
		evaluatedLabel.setText("Behaviour for evaluated agent: ");
		evaluatedLabel.setLocation(20, position.y + 30);

		if (environment.getGoodAgent().getBehaviour() instanceof RandomBehaviour)
		{
			goodLabel.setText(goodLabel.getText().concat("Random"));
			goodLabel.setSize(160, 15);
			evilLabel.setText(evilLabel.getText().concat("Random"));
			evilLabel.setSize(155, 15);
		}
		else if (environment.getGoodAgent().getBehaviour() instanceof PatternBehaviour)
		{
			goodLabel.setText(goodLabel.getText().concat("Pattern: "));
			evilLabel.setText(evilLabel.getText().concat("Pattern: "));
			PatternBehaviour goodBehaviour = (PatternBehaviour) environment.getGoodAgent().getBehaviour();
			PatternBehaviour evilBehaviour = (PatternBehaviour) environment.getEvilAgent().getBehaviour();
	
			JLabel goodLabel2 = new JLabel();
			JLabel goodLabel3 = new JLabel();
			JLabel evilLabel2 = new JLabel();
			JLabel evilLabel3 = new JLabel();
			String goodPattern = goodBehaviour.getPattern();
			String evilPattern = evilBehaviour.getPattern();
	
			int index;
			for (index = 0; index < goodBehaviour.getState(); index++)
				goodLabel.setText(goodLabel.getText().concat(goodPattern.charAt(index) + ""));
			goodLabel.setSize(162 + index * 7 + 5, 15);
			goodLabel2.setText(goodLabel2.getText().concat(goodPattern.charAt(index) + ""));
			goodLabel2.setSize(7 + 5, 15);
			for (index++; index < goodPattern.length(); index++)
				goodLabel3.setText(goodLabel3.getText().concat(goodPattern.charAt(index) + ""));
			goodLabel3.setSize(index * 7, 15);
	
			goodLabel2.setLocation(goodLabel.getLocation().x + goodLabel.getWidth(), position.y);
			goodLabel2.setForeground(new Color(0, 200, 0));
			goodLabel3.setLocation(goodLabel2.getLocation().x + goodLabel2.getWidth(), position.y);
	
			for (index = 0; index < evilBehaviour.getState(); index++)
				evilLabel.setText(evilLabel.getText().concat(evilPattern.charAt(index) + ""));
			evilLabel.setSize(152 + index * 7 + 5, 15);
			evilLabel2.setText(evilLabel2.getText().concat(evilPattern.charAt(index) + ""));
			evilLabel2.setSize(7 + 5, 15);
			for (index++; index < evilPattern.length(); index++)
				evilLabel3.setText(evilLabel3.getText().concat(evilPattern.charAt(index) + ""));
			evilLabel3.setSize(index * 7, 15);
	
			evilLabel2.setLocation(evilLabel.getLocation().x + evilLabel.getWidth(), position.y + 15);
			evilLabel2.setForeground(new Color(255, 0, 0));
			evilLabel3.setLocation(evilLabel2.getLocation().x + evilLabel2.getWidth(), position.y + 15);
	
			add(goodLabel2);
			add(goodLabel3);
			add(evilLabel2);
			add(evilLabel3);
		}

		if (exercise.getEvaluableAgent().getBehaviour() instanceof RandomBehaviour)
		{
			evaluatedLabel.setText(evaluatedLabel.getText().concat("Random"));
			evaluatedLabel.setSize(225, 15);
		}
		else if (exercise.getEvaluableAgent().getBehaviour() instanceof HumanBehaviour)
		{
			evaluatedLabel.setText(evaluatedLabel.getText().concat("Human"));
			evaluatedLabel.setSize(215, 15);
		}
		else if (exercise.getEvaluableAgent().getBehaviour() instanceof TrivialFollowerBehaviour)
		{
			evaluatedLabel.setText(evaluatedLabel.getText().concat("Trivial Follower"));
			evaluatedLabel.setSize(260, 15);
		}
		else if (exercise.getEvaluableAgent().getBehaviour() instanceof OracleBehaviour)
		{
			evaluatedLabel.setText(evaluatedLabel.getText().concat("Oracle"));
			evaluatedLabel.setSize(225, 15);
		}
		else if (exercise.getEvaluableAgent().getBehaviour() instanceof QLearningBehaviour)
		{
			QLearningBehaviour qLearningBehaviour = (QLearningBehaviour) exercise.getEvaluableAgent().getBehaviour();
			evaluatedLabel.setText(evaluatedLabel.getText().concat("Q-Learning - Alfa: " + qLearningBehaviour.getLearningRate() + " - Beta: " + qLearningBehaviour.getRandomExplorationDegree() + " - Gamma: " + qLearningBehaviour.getDiscountFactor() + " - Q0: " + qLearningBehaviour.getInitialQ()));
			evaluatedLabel.setSize(600, 15);
		}

		add(goodLabel);
		add(evilLabel);
		add(evaluatedLabel);
	}
	private void drawNotifyRewardsGeneratorsRelocate(Point position)
	{
		if (environment.getInteractionsToRelocateRewarders() == 0
		 || exercise.getInteractionType().getInteractionNumber() % environment.getInteractionsToRelocateRewarders() != 0
		 || exercise.getInteractionType().getInteractionNumber() == 0)
			return;

		JLabel label = new JLabel();
		label.setText("Reward Generators have been relocated");
		label.setBounds(20, position.y, 250, 15);
		add(label);
	}
	private void drawCellAgents(JPanel panel, Point position, int cellNumber)
	{
		if (!showCellAgents)
			return;

		Cell cell = environment.getSpace().getCell(cellNumber);
		Collection<Agent> agents = cell.getAgents();
		for (Agent agent : agents)
		{
			String name = "";
			if (agent instanceof RewardsGeneratorAgent)
				name += agent.getName().equals("Good") ? InvestigationExercisePanel.GOOD: InvestigationExercisePanel.EVIL;
			else if (agent == this.exercise.getEvaluableAgent())
				name += InvestigationExercisePanel.PI;
			else
				continue;
			JLabel label = new JLabel(name);
			label.setBounds(position.x, position.y, 15, 15);
			panel.add(label);

			if (label.getLocation().getX() + 2 * label.getWidth() + 5 <= panel.getWidth())
				position.x += label.getWidth();
			else
			{
				position.x = 5;
				position.y += label.getHeight() + 5;
			}
		}
	}
	private void drawCellObjects(JPanel panel, Point position, int cellNumber)
	{
		if (!showCellObjects)
			return;

		Cell cell = environment.getSpace().getCell(cellNumber);
		Collection<Objeto> objects = cell.getObjectsNoAgents();
		for (Objeto object : objects)
		{
			JLabel label = new JLabel(object.getName());
			label.setBounds(position.x, position.y, 15, 15);
			panel.add(label);

			if (label.getLocation().getX() + 2 * label.getWidth() + 5 <= panel.getWidth())
				position.x += label.getWidth();
			else
			{
				position.x = 5;
				position.y += label.getHeight() + 5;
			}
		}
	}

	private void initializeParameters()
	{
		showCell = InvestigationExercisePanel.INITIAL_CELL;
		showCellAgents = InvestigationExercisePanel.INITIAL_CELL_AGENTS;
		showCellObjects = InvestigationExercisePanel.INITIAL_CELL_OBJECTS;
		showMovements = InvestigationExercisePanel.INITIAL_MOVEMENTS;
		showLastReward = InvestigationExercisePanel.INITIAL_LAST_REWARD;
		showTotalReward = InvestigationExercisePanel.INITIAL_TOTAL_REWARD;
	}
}