package observer;


import java.util.ArrayList;


/**
 * Representa a un sujeto dentro del patrn Observador
 * 
 * @author Javier
 * @since 31/03/2011
 */
public abstract class Subject
{
	// Atributos del objeto
	/**
	 * Observadores que siguen a este sujeto
	 */
	private ArrayList<Observer> observerCollection;


	// Constructores
	public Subject()
	{
		observerCollection = new ArrayList<Observer>();
	}

	// Funciones del objeto
	/**
	 * Registra un nuevo observador para que sea notificado
	 */
	public void registerObserver(Observer observer)
	{
		observerCollection.add(observer);
	}
	/**
	 * Elimina un observador para que no vuelva a ser notificado
	 * @param observer
	 */
	public void unregisterObserver(Observer observer)
	{
		observerCollection.remove(observer);
	}
	/**
	 * Elimina todos los observadores para que no vuelvan a ser notificados
	 */
	public void unregisterObservers()
	{
		observerCollection.clear();
	}
	/**
	 * Notificar a todos los observadores que el objeto a cambiado
	 */
	public void notifyObservers()
	{
		for (Observer observer : observerCollection)
			observer.notifyObserver();
	}
}