package files;


import java.io.File;
import java.io.FileWriter;
import java.io.PrintWriter;

import anynt.environment.LambdaEnvironment;
import anynt.environment.settings.NoShareRewards;
import anynt.environment.settings.ShareRewardsBetweenEvaluableAgents;
import anynt.exercise.Exercise;
import anynt.object.Agent;
import anynt.object.EvaluableAgent;
import anynt.object.behaviour.AgentBehaviour;
import anynt.object.behaviour.HumanBehaviour;
import anynt.object.behaviour.OracleBehaviour;
import anynt.object.behaviour.PatternBehaviour;
import anynt.object.behaviour.QLearningBehaviour;
import anynt.object.behaviour.RandomBehaviour;
import anynt.object.behaviour.TrivialFollowerBehaviour;
import anynt.object.moveController.NumberOfMovementsMoveController;
import anynt.object.moveController.PatternMoveController;


public class ExperimentsFile
{
	public static String FILE_PATH_DEFINED = "C:/Users/Javier/Desktop/Sample environment with 8 Cells.csv";
	public static String FILE_PATH_GENERATED = "C:/Users/Javier/Desktop/Generated with X Cells.csv";
	public static String FILE_PATH_SOCIAL = "C:/Users/Javier/Desktop/Social with X Cells.csv";
	public static String FILE_PATH_REACTIVE = "C:/Users/Javier/Desktop/Reactive with X Cells.csv";

	public static boolean WriteExperiment(Exercise exercise, String filePath)
	{
		// Take the environment of the exercise
		LambdaEnvironment environment = (LambdaEnvironment) exercise.getEnvironment();
		boolean exportationSuccessfull = true;
		FileWriter fileWriter = null;
		PrintWriter printWriter = null;

		File file = new File(filePath);
		file.exists();

		try
		{
			fileWriter = new FileWriter(filePath, true);
			printWriter = new PrintWriter(fileWriter);

			// This represent the structure of all the lines exported to the document.
			/*{
				printWriter.print("\"Space Cells\"");
				printWriter.print(";\"Space Actions\"");
				printWriter.print(";\"Space Description\"");
				printWriter.print(";\"Rewarders Actions per Step\"");
				printWriter.print(";\"Rewarders Fixed Actions\"");
				printWriter.print(";\"Rewarders Behaviour\"");
				printWriter.print(";\"Pattern Description\"");
				printWriter.print(";\"Pattern Length\"");
				printWriter.print(";\"Rewarders Relocation\"");
				printWriter.print(";\"Agent\"");
				printWriter.print(";\"Session Interactions\"");
				printWriter.print(";\"Session Consumption Order\"");
				printWriter.print(";\"Session Reward Decreasing Factor\"");
				printWriter.print(";\"Session Consume Rewards\"");
				printWriter.print(";\"Session Share Rewards\"");
				printWriter.print(";\"Total Reward\"");

				printWriter.println();
			}*/

			// Export the results for every agent
			for (int agentIndex = 0; agentIndex < environment.getAgents().size(); agentIndex++)
			{
				// Take the agent and only export the results if it's a evaluable agent
				Agent agent = environment.getAgents().get(agentIndex);
				if (!(agent instanceof EvaluableAgent))
					continue;

				// Number of space sells
				printWriter.print(environment.getSpace().getNumberOfCells());
				// Number of space actions
				printWriter.print(";" + environment.getSpace().getNumberOfActions());
				// Space Description
				printWriter.print(";" + environment.getSpace().getDescription());
				// Behavior of rewarders agents
				if (environment.getGoodAgent().getMoveController() instanceof NumberOfMovementsMoveController)
				{
					// Rewarders Actions per Step
					printWriter.print(";\"N Movements\"");
					// Rewarders Fixed Actions
					printWriter.print(";" + ((NumberOfMovementsMoveController) environment.getGoodAgent().getMoveController()).getNumberOfMovements());
					// Rewarders Behaviour
					if (environment.getGoodAgent().getBehaviour() instanceof RandomBehaviour)
						printWriter.print(";\"Random\"");
					else if (environment.getGoodAgent().getBehaviour() instanceof PatternBehaviour)
						printWriter.print(";\"Pattern\"");
					// Pattern Description
					printWriter.print(";\"p" + ((PatternBehaviour)environment.getGoodAgent().getBehaviour()).getPattern() + "\"");
					// Pattern Length
					printWriter.print(";" + ((PatternBehaviour)environment.getGoodAgent().getBehaviour()).patternLenght());
				}
				else if (environment.getGoodAgent().getMoveController() instanceof PatternMoveController)
				{
					// Rewarders Actions per Step
					printWriter.print(";\"Pattern\"");
					// Rewarders Fixed Actions
					printWriter.print(";");
					// Rewarders Behaviour
					printWriter.print(";");
					// Pattern Description
					printWriter.print(";");
					// Pattern Length
					printWriter.print(";");
				}
				else
				{
					// Rewarders Actions per Step
					printWriter.print(";");
					// Rewarders Fixed Actions
					printWriter.print(";");
					// Rewarders Behaviour
					printWriter.print(";");
					// Pattern Description
					printWriter.print(";");
					// Pattern Length
					printWriter.print(";");
				}

				// Number of interactions when rewarders where relocated
				printWriter.print(";" + environment.getInteractionsToRelocateRewarders());

				// What kind of evaluable agent was doing the exercise
				AgentBehaviour agentBehaviour = agent.getBehaviour();
				if (agentBehaviour instanceof TrivialFollowerBehaviour)
					printWriter.print(";\"TrivialFollower\"");
				else if (agentBehaviour instanceof RandomBehaviour)
					printWriter.print(";\"Random\"");
				else if (agentBehaviour instanceof OracleBehaviour)
					printWriter.print(";\"Oracle\"");
				else if (agentBehaviour instanceof QLearningBehaviour)
					printWriter.print(";\"QLearning\"");
				else if (agentBehaviour instanceof HumanBehaviour)
					printWriter.print(";\"Human\"");
				else
					printWriter.print(";");

				// Number of interactions made in the session
				printWriter.print(";" + exercise.getInteractionType().getInteractionNumber());
				// Consumption order of the rewards (before or after update)
				if (environment.getRewardsConsumedBeforeRewardersUpdateCellRewards())
					printWriter.print(";\"Before\"");
				else
					printWriter.print(";\"After\"");
				// Session Reward Decreasing Factor
				printWriter.print(";" + environment.getDecreaseFactor());
				// Evaluable agents consume rewards
				if (environment.getEvaluableAgentConsumeRewards())
					printWriter.print(";\"Yes\"");
				else
					printWriter.print(";\"No\"");
				// Kind of rewards sharing
				if (environment.getRewardsSharing() instanceof NoShareRewards)
					printWriter.print(";\"No\"");
				else if (environment.getRewardsSharing() instanceof ShareRewardsBetweenEvaluableAgents)
					printWriter.print(";\"Between Evaluable Agents\"");
				else
					printWriter.print(";\"Other\"");

				// The total reward of the agent
				String totalReward = Double.toString(((EvaluableAgent) agent).totalReward());
				totalReward = totalReward.replace('.', ',');
				printWriter.print(";" + totalReward);

				// All the rewards taken on each interaction, from first interaction to last interaction
				for (int interaction = 1; interaction <= exercise.getInteractionType().getInteractionNumber(); interaction++)
				{
					String reward = Double.toString(agent.getInteractionHistory().get(interaction).getReward());
					reward = reward.replace('.', ',');
					printWriter.print(";" + reward);
				}

				printWriter.println();
			}
		}
		catch (Exception e)
		{
			exportationSuccessfull = false;
			e.printStackTrace();
		}
		finally
		{
			try
			{
				fileWriter.close();
			}
			catch (Exception e2)
			{
				e2.printStackTrace();
			}
		}

		return exportationSuccessfull;
	}
}